@@ -13,6 +13,8 @@ using Gtk;
 using GtkSharp;
 using Gnome;
 
+using Mono.Posix;
+
 using Beagle;
 using Beagle.Util;
 
@@ -20,7 +22,7 @@ namespace Best {
 	
 	class Best {
 
-		static public string DefaultWindowTitle = "Beagle Search (alpha)";
+		static public string DefaultWindowTitle = "Beagle Search (beta)";
 
 		static void PrintUsageAndExit ()
 		{
@@ -44,6 +46,7 @@ namespace Best {
 		static string query = "";
 		static bool doTray = true;
 		static bool showWindow = false;
+		static bool autostarted = false;
 
 		static void ParseArgs (String[] args)
 		{
@@ -72,10 +75,20 @@ namespace Best {
 					i++;
 					break;
 
+				case "--autostarted":
+					if (! Conf.Searching.Autostart){
+						Console.WriteLine ("Autostarting is disabled, not starting");
+                                                Environment.Exit (0);
+					}
+					autostarted = true;
+					break;
+
 				default:
-					if (query.Length != 0)
-						query += " ";
-					query += args [i];
+					if (args [i].Length < 2 || args [i].Substring (0, 2) != "--") {
+						if (query.Length != 0)
+							query += " ";
+						query += args [i];
+					}
 					break;
 				}
 
@@ -97,6 +110,9 @@ namespace Best {
 			GeckoUtils.Init ();
 			GeckoUtils.SetSystemFonts ();
 
+			// I18N
+			Catalog.Init ("beagle", ExternalStringsHack.LocaleDir);
+
 			// Create the window.
 			BestWindow win;
 			if (query != "") {
@@ -111,9 +127,10 @@ namespace Best {
 			// Create the tray icon.
 			BestTray icon;			
 			if (doTray) {
-				icon = new BestTray (win);
-				Console.WriteLine ("If you're wondering whether Best is working check " +
-						   "your notification area (system tray)");
+				icon = new BestTray (win, autostarted);
+				icon.Show ();
+				Console.WriteLine (Catalog.GetString ("If you're wondering whether Best is working check " +
+						   "your notification area (system tray)"));
 			} else {
 				win.Show ();
 				win.Present ();
