@@ -27,11 +27,12 @@
 // DEALINGS IN THE SOFTWARE.
 //
 
-
 using System;
 using System.IO;
 using System.Runtime.InteropServices;
+
 using Beagle.Util;
+using Beagle.Daemon;
 
 namespace Beagle.Filters {
     
@@ -41,9 +42,13 @@ namespace Beagle.Filters {
 
 		//////////////////////////////////////////////////////////
 
-		private delegate void TextHandlerCallback (IntPtr byteArray, int dataLen, bool hotText);
-		//private delegate void TextHandlerCallback (string data, int dataLen, bool hotText);
+		private delegate void TextHandlerCallback (IntPtr byteArray, int dataLen, 
+							   IntPtr byteHotArray, int hotDataLen,
+							   bool appendStructBrk);
 		
+		[DllImport ("wv1glue")]
+		private static extern int wv1_init ();
+
 		[DllImport ("wv1glue")]
 		private static extern int wv1_glue_init_doc_parsing (string fname, TextHandlerCallback callback);
 
@@ -100,27 +105,50 @@ namespace Beagle.Filters {
 
 		//////////////////////////////////////////////////////////
 
+		static bool wv1_Initted = false;
+
 		public FilterDOC () 
 		{
-			AddSupportedMimeType ("application/msword");
-			AddSupportedMimeType ("application/vnd.ms-word");
-			AddSupportedMimeType ("application/x-msword");
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/msword"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/vnd.ms-word"));
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/x-msword"));
 			SnippetMode = true;
 		}
 		
-  		private void IndexText (IntPtr byteArray, int dataLen, bool hotText)
+  		private void IndexText (IntPtr byteArray, int dataLen, 
+					IntPtr byteHotArray, int hotDataLen,
+					bool appendStructBrk)
   		{
-			if (byteArray != IntPtr.Zero) {
-				byte[] data = new byte[dataLen];
-				Marshal.Copy (byteArray, data, 0, dataLen);
-				if (hotText)
-					HotUp();
-				AppendText (System.Text.Encoding.UTF8.GetString(data, 0, dataLen));
+			byte[] data = null;
+			string str = null;
+			string strHot = null;
+
+			try {
+				if (dataLen > 0){
+					data = new byte[dataLen];
+					Marshal.Copy (byteArray, data, 0, dataLen);
+				}
 			
-				if (hotText)
-					HotDown();
+				if (data != null)
+					str = System.Text.Encoding.UTF8.GetString (data, 0, dataLen);
+
+				data = null;
+				if (hotDataLen > 0) {
+					data = new byte [hotDataLen];
+					Marshal.Copy (byteHotArray, data, 0, hotDataLen);
+				}
+				if (data != null)
+					strHot = System.Text.Encoding.UTF8.GetString (data, 0, hotDataLen);
+			
+				AppendText (str, strHot);
+			
+				if (appendStructBrk)
+					AppendStructuralBreak ();
+			} catch (Exception e) {
+				Logger.Log.Debug ("Exception occurred in Word-Doc filter. {0}", e);
 			}
   		}
+
 		override protected void DoOpen (FileInfo info)
 		{
 			FileName = info.FullName;
@@ -188,13 +216,11 @@ namespace Beagle.Filters {
 			
 			intProp = wv1_glue_get_page_count (oleSummaryStream);
 			if (intProp != 0)	
-				AddProperty (Beagle.Property.New ("fixme:page-count", intProp ));
-			intProp = 0;
+				AddProperty (Beagle.Property.New ("fixme:page-count", intProp.ToString ()));
 			
 			intProp = wv1_glue_get_word_count (oleSummaryStream);
 			if (intProp != 0)
-				AddProperty (Beagle.Property.New ("fixme:word-count", intProp));
-			intProp = 0;
+				AddProperty (Beagle.Property.New ("fixme:word-count", intProp.ToString ()));
 
 			wv1_glue_close_stream (oleStream, oleSummaryStream);
 		}
@@ -204,15 +230,24 @@ namespace Beagle.Filters {
 			int ret;
 			TextHandlerCallback textHandler;
 			textHandler = new TextHandlerCallback (IndexText);
-			
-			ret = wv1_glue_init_doc_parsing (FileName, textHandler);
 
+			if (!wv1_Initted) {
+				wv1_init ();
+				wv1_Initted = true;
+			}
+
+			Stopwatch stopwatch = new Stopwatch ();
+			stopwatch.Start ();
+
+			ret = wv1_glue_init_doc_parsing (FileName, textHandler);
 			if (ret == -2)
 				Logger.Log.Error ("{0} : is password protected", FileName);
 			else if (ret == -1)
 				Logger.Log.Error ("{0} : Unable to read", FileName);
 			else if (ret == -3)
 				Logger.Log.Error ("Unable to initiate the parser for {0}", FileName);
+			stopwatch.Stop ();
+			Logger.Log.Info ("Word document extraction done in {0}", stopwatch);
 			Finished ();
 		}
 	}
