@@ -1,7 +1,7 @@
 //
 // FilterSource.cs
 //
-// Copyright (C) 2004 Novell, Inc.
+// Copyright (C) 2004, 2005 Novell, Inc.
 //
 
 //
@@ -39,7 +39,9 @@ namespace Beagle.Filters {
 			C_Sharp_Style,
 			Python_Style,
 			Fortran_Style,
-			Pascal_Style
+			Pascal_Style,
+			Lisp_Style,
+			Matlab_Style
 		};
 		
 		protected LangType SrcLangType;
@@ -72,7 +74,7 @@ namespace Beagle.Filters {
 		// tokens for indexing.
 		//
 		// FIXME: Perl has embedded "POD" (documentation) style, which needs a little processing.
-		//        Pascal has its own style of comments.
+		//
  		protected void ExtractTokens (string str)
 		{
 			int index;
@@ -80,14 +82,61 @@ namespace Beagle.Filters {
 			string splCharSeq = "";
 
 			for (index = 0; index < str.Length; index++) {
-				if ((str[index] == '/' || str[index] == '*') &&
-				    (SrcLangType == LangType.C_Style ||
-				     SrcLangType == LangType.C_Sharp_Style ||
-				     SrcLangType == LangType.Pascal_Style)) {		
+		         	if (((str[index] == '{' 
+				      || str[index] == '}' 
+				      || str[index] == '(' 
+				      || str[index] == ')'
+				      || str[index] == '*'
+				      || str[index] == '/')
+				     && SrcLangType == LangType.Pascal_Style)
+				    || ((str[index] == '/' 
+					 || str[index] == '*') 
+					&& (SrcLangType == LangType.C_Style 
+					    || SrcLangType == LangType.C_Sharp_Style))) {		
+					
 					splCharSeq += str[index];
 					
 					switch (splCharSeq) {
-						
+
+                                        case "(*":
+                                                if (SrcLineType == LineType.None) {
+							SrcLineType = LineType.BlockComment;
+							token.Remove (0, token.Length);
+						    } else 
+							token.Append (splCharSeq);
+						splCharSeq = ""; 
+                                                break;
+
+					case "*)":
+						if (SrcLineType == LineType.BlockComment) {
+							SrcLineType = LineType.None;
+							token.Append (" ");
+							AppendText (token.ToString());
+							token.Remove (0, token.Length);
+						} else if (SrcLineType != LineType.None)
+							token.Append (splCharSeq);
+						splCharSeq = "";
+						break;          
+                           
+                                        case "{":
+                                                if (SrcLineType == LineType.None) {
+							SrcLineType = LineType.BlockComment;
+							token.Remove (0, token.Length);
+						    } else 
+							token.Append (splCharSeq);
+						splCharSeq = ""; 
+                                                break;
+
+					case "}":
+						if (SrcLineType == LineType.BlockComment) {
+							SrcLineType = LineType.None;
+							token.Append (" ");
+							AppendText (token.ToString());
+							token.Remove (0, token.Length);
+						} else if (SrcLineType != LineType.None)
+							token.Append (splCharSeq);
+						splCharSeq = "";
+						break;
 					case "//":
 						if (SrcLineType == LineType.None) {
 							SrcLineType = LineType.SingleLineComment;
@@ -117,8 +166,10 @@ namespace Beagle.Filters {
 						splCharSeq = "";
 						break;
 					}
-				} else if (str[index] == '#' && SrcLangType == LangType.Python_Style ||
-					   str[index] == '!' && SrcLangType == LangType.Fortran_Style) {
+				} else if ((str[index] == '#' && SrcLangType == LangType.Python_Style) ||
+					   (str[index] == '!' && SrcLangType == LangType.Fortran_Style) ||
+					   (str[index] == ';' && SrcLangType == LangType.Lisp_Style) ||
+					   (str[index] == '%' && SrcLangType == LangType.Matlab_Style)) {
 					if (SrcLineType == LineType.None) {
 						SrcLineType = LineType.SingleLineComment;
 						token.Remove (0, token.Length);
@@ -155,6 +206,10 @@ namespace Beagle.Filters {
 					       
 					splCharSeq = "";
 				}
+				// Lisp: ignore the single quote character; do another iteration
+				else if (SrcLangType == LangType.Lisp_Style && str[index] == '\'') {
+					continue;
+				}
 				else if (str[index] == '\"' || str[index] == '\'') {
 
 					if (SrcLineType == LineType.StringConstant &&
@@ -181,9 +236,23 @@ namespace Beagle.Filters {
 				} else if (SrcLineType == LineType.None) {
 					if (Char.IsLetter (str[index]) ||
 					    Char.IsDigit (str[index]) ||
-					    str[index] == '_')
+					    str[index] == '_') {
 						token.Append (str[index]);
-					else {
+					} else {
+						if (SrcLangType == LangType.Lisp_Style) {
+
+							// Lisp identifiers: letters, digits, and:
+							// ! $ % & * + - . / : < = > ? @ ^ _ ~
+							switch (str[index]) {
+							case '!': case '$': case '%': case '&':
+							case '*': case '+': case '-': case '.':
+							case '/': case ':': case '<': case '=':
+							case '>': case '?': case '@': case '^':
+							case '_': case '~':
+								token.Append (str[index]);
+								continue;
+							}
+						}
 						token = token.Replace(" ", "");
 						if (token.Length > 0) {
 							string tok = token.ToString(); 
@@ -203,10 +272,14 @@ namespace Beagle.Filters {
 				}
 		        }
 			if (SrcLineType != LineType.None) {
+				bool trailing_backslash = false;
+
 				token.Append (splCharSeq);
 
-				if (token.Length > 0 && token [token.Length - 1] == '\\')
+				if (token.Length > 0 && token [token.Length - 1] == '\\') {
 					token = token.Remove (token.Length-1, 1);
+					trailing_backslash = true;
+				}
 			       
 				token.Append (" ");
 				AppendText (token.ToString());
@@ -214,16 +287,17 @@ namespace Beagle.Filters {
 				// if a single-line-comment ends with a "\", 
 				// the lines that follows it are also considered as a comment,
 				// till a line with out a "\" is found
-				// C# doesn't follow this syntax.
-				if (SrcLangType == LangType.C_Sharp_Style ||
-				    (SrcLineType == LineType.SingleLineComment &&
-				     str.Length > 0 && str[str.Length - 1] != '\\'))
-					SrcLineType = LineType.None;
-			} else if (SrcLangType == LangType.Python_Style) {
-				if (token.Length > 0 && !Char.IsDigit (token[0])) {
-					token.Append (" ");
-					AppendText (token.ToString());
-				}
+				// C# and Lisp don't follow this syntax.
+				if (SrcLineType == LineType.SingleLineComment) 
+					if (!trailing_backslash 
+					    || SrcLangType == LangType.C_Sharp_Style
+					    || SrcLangType == LangType.Lisp_Style)
+						SrcLineType = LineType.None;
+			} else if (token.Length > 0 
+				   && !Char.IsDigit (token[0])) { 
+				/* we don't want any numeric const */
+				token.Append (" ");
+				AppendText (token.ToString());
 			}
 		}
 	}
