@@ -27,27 +27,32 @@ namespace ImLogViewer {
 		[Widget] Button clear_button;
 		[Widget] TextView conversation;
 		
-		private string first_selected_log;
-		private bool have_first_selection_iter = false;
-		private TreeIter first_selection_iter;
+		private string selected_log;
+
 		private string speaking_to;
 		private string log_path;
 		private string highlight_text;
 		private string search_text;
 
 		private TreeStore tree_store;
-
 		private ThreadNotify index_thread_notify;
 		private Timeline timeline = new Timeline ();
 
+		private string client;
+
 		public ImLogWindow (string path, string search, string highlight)
 		{
+			// FIXME: This is a bunch of crap, ImLog should be more dynamic.
+			if (path.IndexOf (".gaim/logs") != -1)
+				client = "gaim";
+			else if (path.IndexOf ("apps/kopete/logs") != -1)
+				client = "kopete";
+			
 			if (Directory.Exists (path)) {
 				log_path = path;
-				first_selected_log = null;
 			} else if (File.Exists (path)) {
 				log_path = Path.GetDirectoryName (path);
-				first_selected_log = path;
+				selected_log = path;
 			} else {
 				Console.WriteLine ("ERROR: Log path doesn't exist - {0}", path);
 				return;
@@ -71,7 +76,12 @@ namespace ImLogViewer {
 				return;
 
 			// Find the buddy
-			ImBuddy buddy = new GaimBuddyListReader ().Search (speaker);
+			ImBuddy buddy = null;
+
+			if (client == "gaim")
+				buddy = new GaimBuddyListReader ().Search (speaker);
+			else if (client == "kopete")
+				buddy = new KopeteBuddyListReader ().Search (speaker);
 			
 			if (speaker.EndsWith (".chat")) {
 				imviewer.Title = String.Format (Catalog.GetString ("Conversations in {0}"), speaker.Replace (".chat", ""));
@@ -94,8 +104,6 @@ namespace ImLogViewer {
 			Glade.XML gxml = new Glade.XML (null, "ImLogViewer.glade", "imviewer", null);
 			gxml.Autoconnect (this);
 
-			//BindKeys ();
-
 			conversation.PixelsAboveLines = 3;
 			conversation.LeftMargin = 4;
 			conversation.RightMargin = 4;
@@ -142,8 +150,16 @@ namespace ImLogViewer {
 		private void IndexLogs ()
 		{
 			foreach (string file in Directory.GetFiles (log_path)) {
-				ICollection logs = GaimLog.ScanLog (new FileInfo (file));
+				ICollection logs = null;
 				
+				if (client == "gaim")
+					logs = GaimLog.ScanLog (new FileInfo (file));
+				else if (client == "kopete")
+					logs = KopeteLog.ScanLog (new FileInfo (file));
+				
+				if (logs == null)
+					continue;
+
 				foreach (ImLog log in logs) {
 					if (speaking_to == null)
 						SetWindowTitle (log.SpeakingTo);
@@ -188,7 +204,7 @@ namespace ImLogViewer {
 			}
 		}
 
-		private void DoFoo (ArrayList list, string name, string date_format)
+		private void AddCategory (ArrayList list, string name, string date_format)
 		{
 			if (list.Count > 0) {
 				ArrayList previews = GetPreviews (list);
@@ -203,29 +219,27 @@ namespace ImLogViewer {
 		{
 			tree_store.Clear ();
 
-			DoFoo (timeline.Today, "Today", "HH:mm");
-			DoFoo (timeline.Yesterday, "Yesterday", "HH:mm");
-			DoFoo (timeline.ThisWeek, "This Week", "dddd");
-			DoFoo (timeline.LastWeek, "Last Week", "dddd");
-			DoFoo (timeline.ThisMonth, "This Month", "MMM d");
-			DoFoo (timeline.ThisYear, "This Year", "MMM d");
-			DoFoo (timeline.Older, "Older", "yyy MMM d");
+			AddCategory (timeline.Today, "Today", "HH:mm");
+			AddCategory (timeline.Yesterday, "Yesterday", "HH:mm");
+			AddCategory (timeline.ThisWeek, "This Week", "dddd");
+			AddCategory (timeline.LastWeek, "Last Week", "dddd");
+			AddCategory (timeline.ThisMonth, "This Month", "MMM d");
+			AddCategory (timeline.ThisYear, "This Year", "MMM d");
+			AddCategory (timeline.Older, "Older", "yyy MMM d");
 		
 			timelinetree.ExpandAll();
 		}
 
 		private void AddPreviews (TreeIter parent, ArrayList previews, string date_format)
 		{
-			TreeIter iter;
-
 			foreach (ImLogPreview preview in previews) {
 				string date = preview.Log.StartTime.ToString (date_format);
-				iter = tree_store.AppendValues (parent, date, preview.Snippet, preview.Log);
-				if (! have_first_selection_iter || preview.Log.LogFile == first_selected_log) {
-					have_first_selection_iter = true;
-					first_selection_iter = iter;
-					timelinetree.Selection.SelectIter (first_selection_iter);
+				tree_store.AppendValues (parent, date, preview.Snippet, preview.Log);
+	
+				if (selected_log == null || selected_log == preview.Log.LogFile) {
+					selected_log = preview.Log.LogFile;
 					RenderConversation (preview.Log);
+					ScrollToLog (preview.Log.LogFile);
 				}
 			}
 		}
@@ -249,7 +263,10 @@ namespace ImLogViewer {
  		private void RenderConversation (ImLog im_log)
  		{
 			TextBuffer buffer = conversation.Buffer;
-			buffer.Delete (buffer.StartIter, buffer.EndIter);
+			TextIter start = buffer.StartIter;
+			TextIter end = buffer.EndIter;
+
+			buffer.Delete (ref start, ref end);
 			
  			if (im_log == null) {
  				//SetStatusTitle (new DateTime ());
@@ -260,9 +277,11 @@ namespace ImLogViewer {
 
 			TextTag bold = buffer.TagTable.Lookup ("bold");
 
+			end = buffer.EndIter;
+
  			foreach (ImLog.Utterance utt in im_log.Utterances) {
-				buffer.InsertWithTags (buffer.EndIter, utt.Who + ":", new TextTag[] {bold});
-				buffer.Insert (buffer.EndIter, String.Format(" {0}\n", utt.Text));
+				buffer.InsertWithTags (ref end, utt.Who + ":", new TextTag[] {bold});
+				buffer.Insert (ref end, String.Format(" {0}\n", utt.Text));
 			}
 
 			if (highlight_text != null)
@@ -284,7 +303,7 @@ namespace ImLogViewer {
 				if (word == String.Empty)
 					continue;
 
-				while ((idx = text.IndexOf (word.ToLower (), idx)) != -1) {					
+				while ((idx = text.IndexOf (word.ToLower (), idx)) != -1) {
 					Gtk.TextIter start = buffer.GetIterAtOffset (idx);
 					Gtk.TextIter end = start;
 					end.ForwardChars (word.Length);
@@ -305,7 +324,7 @@ namespace ImLogViewer {
 
 			search_text = text;
 			highlight_text = null;
-			have_first_selection_iter = false;
+			selected_log = null;
 		}
 
 		private void OnConversationSelected (object o, EventArgs args) 
@@ -315,6 +334,11 @@ namespace ImLogViewer {
 			
 			if (((TreeSelection)o).GetSelected (out model, out iter)) {
 				ImLog log = model.GetValue (iter, 2) as ImLog;
+
+				if (log == null)
+					return;
+
+				selected_log = log.LogFile;
 				RenderConversation (log);
 			}
 		}
@@ -331,9 +355,37 @@ namespace ImLogViewer {
 
 		private void OnSearchClicked (object o, EventArgs args)
 		{
+			if (search_entry.Text == null || search_entry.Text == "")
+				return;
+
 			Search (search_entry.Text);
 			RepopulateTimeline ();
 		}
+
+		private void ScrollToLog (string scroll_log)
+		{
+			TreeIter root_iter;
+			tree_store.GetIterFirst (out root_iter);
+			
+			do {
+				if (tree_store.IterHasChild (root_iter)) {
+					TreeIter child;
+					tree_store.IterNthChild (out child, root_iter, 0);
+					
+					do {
+						ImLog log = tree_store.GetValue (child, 2) as ImLog;
+						
+						if (log.LogFile == scroll_log) {
+							TreePath path = tree_store.GetPath (child);
+							timelinetree.ExpandToPath (path);
+							timelinetree.Selection.SelectPath (path);
+							timelinetree.ScrollToCell (path, null, true, 0.5f, 0.0f);
+							return;
+						}
+					} while (tree_store.IterNext (ref child));
+				}
+			} while (tree_store.IterNext (ref root_iter));
+		}
 		
 		private void OnClearClicked (object o, EventArgs args)
 		{
@@ -343,6 +395,8 @@ namespace ImLogViewer {
 			search_entry.Sensitive = true;
 
 			RepopulateTimeline ();
+
+			ScrollToLog (selected_log);
 		}
 	}
 }
