@@ -35,6 +35,8 @@ namespace Beagle.Tile {
 
 	[HitFlavor (Name="Folders", Rank=600, Emblem="emblem-folder.png", Color="#f5f5fe",
 		    Type="File", Uri="file://*", MimeType="inode/directory")]
+	[HitFlavor (Name="Folders", Rank=600, Emblem="emblem-folder.png", Color="#f5f5fe",
+		    Type="File", Uri="file://*", MimeType="x-directory/normal")]
 	public class TileFolder : TileFromHitTemplate {
 		public TileFolder (Hit _hit) : base (_hit, 
 						     "template-folder.html")
@@ -53,12 +55,14 @@ namespace Beagle.Tile {
 				str = String.Format (Catalog.GetPluralString ("Contains {0} Item", "Contains {0} Items", n), n);
 
 			Template["Contents"] = str;
+
+			Template ["Icon"] = (Hit ["fixme:Icon"] != null && File.Exists (Hit ["fixme:Icon"])) ? Images.GetHtmlSource (Hit ["fixme:Icon"], null) : Images.GetHtmlSourceForStock ("gnome-fs-directory", 48);
 		}
 
 		[TileAction]
 		public override void Open ()
 		{
-			OpenFromMime (Hit, "nautilus", "--no-desktop", true);
+			OpenFolder (Hit.Uri.LocalPath);
 		} 
 
                 [TileAction]
