@@ -46,10 +46,10 @@ namespace Beagle.Util {
 		public static DaemonConfig Daemon = null;
 		public static SearchingConfig Searching = null;
 
-#if ENABLE_WEBSERVICES		
+//#if ENABLE_WEBSERVICES		
 		public static NetworkingConfig Networking = null;
 		public static WebServicesConfig WebServices = null;
-#endif 		
+//#endif 		
 		private static string configs_dir;
 		private static Hashtable mtimes;
 		private static Hashtable subscriptions;
@@ -63,9 +63,9 @@ namespace Beagle.Util {
 
 		static Conf ()
 		{
-			Sections = new Hashtable (1);
-			mtimes = new Hashtable (1);
-			subscriptions = new Hashtable (1);
+			Sections = new Hashtable (3);
+			mtimes = new Hashtable (3);
+			subscriptions = new Hashtable (3);
 
 			configs_dir = Path.Combine (PathFinder.StorageDir, "config");
 			if (!Directory.Exists (configs_dir))
@@ -148,7 +148,7 @@ namespace Beagle.Util {
 		        Searching = (SearchingConfig) temp;
 			NotifySubscribers (Searching);
 
-#if ENABLE_WEBSERVICES
+//#if ENABLE_WEBSERVICES
 			LoadFile (typeof (NetworkingConfig), Networking, out temp, force);
 		    	Networking = (NetworkingConfig) temp;
 			NotifySubscribers (Networking);
@@ -156,7 +156,7 @@ namespace Beagle.Util {
 			LoadFile (typeof (WebServicesConfig), WebServices, out temp, force);
 		    	WebServices = (WebServicesConfig) temp;
 			NotifySubscribers (WebServices);
-#endif
+//#endif
 
 			watching_for_updates = true;
 		}
@@ -188,8 +188,9 @@ namespace Beagle.Util {
 					ConstructDefaultSection (type, sectionname, out section);
 				return false;
 			}
+
 			if (!force && current != null && mtimes.ContainsKey (sectionname) &&
-					File.GetLastWriteTimeUtc (filepath).CompareTo ((DateTime) mtimes [sectionname]) < 0)
+					File.GetLastWriteTimeUtc (filepath).CompareTo ((DateTime) mtimes [sectionname]) <= 0)
 				return false;
 
 			Logger.Log.Debug ("Loading {0} from {1}", type, filename);
@@ -234,7 +235,7 @@ namespace Beagle.Util {
 				watching_for_updates = false;
 				fs = new FileStream (filepath, FileMode.Create);
 				XmlSerializer serializer = new XmlSerializer (type);
-				serializer.Serialize (fs, section);
+				XmlFu.SerializeUtf8 (serializer, fs, section);
 			} catch (Exception e) {
 				if (fs != null)
 					fs.Close ();
@@ -309,8 +310,14 @@ namespace Beagle.Util {
 
 		[ConfigSection (Name="searching")]
 		public class SearchingConfig : Section {
+			
+			private bool autostart = true;
+			public bool Autostart {
+				get { return autostart; }
+				set { autostart = value; }
+			}
+			
 			private KeyBinding show_search_window_binding = new KeyBinding ("F12");
-
 			public KeyBinding ShowSearchWindowBinding {
 				get { return show_search_window_binding; }
 				set { show_search_window_binding = value; }
@@ -337,6 +344,13 @@ namespace Beagle.Util {
 				set { denied_backends = value; }
 			}
 
+			private bool index_synchronization = true;
+			public bool IndexSynchronization {
+				get { return index_synchronization; }
+				// Don't really want to expose this, but serialization requires it
+				set { index_synchronization = value; }
+			}
+
 			[ConfigOption (Description="Add a static queryable", Params=1, ParamsDescription="Index path")]
 			internal bool AddStaticQueryable (out string output, string [] args)
 			{
@@ -361,6 +375,14 @@ namespace Beagle.Util {
 					output += String.Format (" - {0}\n", index_path);
 				return true;
 			}
+
+			[ConfigOption (Description="Toggles whether your indexes will be synchronized locally if your home directory is on a network device (eg. NFS/Samba)")]
+			internal bool ToggleIndexSynchronization (out string output, string [] args)
+			{
+				index_synchronization = !index_synchronization;
+				output = "Index Synchronization is " + ((index_synchronization) ? "enabled" : "disabled") + ".";
+				return true;
+			}		
 		}
 
 		[ConfigSection (Name="indexing")]
@@ -435,11 +457,49 @@ namespace Beagle.Util {
 					output += String.Format (" - [{0}] {1}\n", exclude_item.Type.ToString (), exclude_item.Value);
 				return true;
 			}
-		
-			// FIXME: Add methods to manipulate excludes
+
+			[ConfigOption (Description="Add a resource to exclude from indexing", Params=2, ParamsDescription="A type [path/pattern/mailfolder], a path/pattern/name")]
+			internal bool AddExclude (out string output, string [] args)
+			{
+				ExcludeType type;
+				try {
+					type = (ExcludeType) Enum.Parse (typeof (ExcludeType), args [0], true);
+				} catch (Exception e) {
+					output = String.Format("Invalid type '{0}'. Valid types: Path, Pattern, MailFolder", args [0]);
+					return false;
+				}
+
+				excludes.Add (new ExcludeItem (type, args [1]));
+				output = "Exclude added.";
+				return true;
+			}
+
+			[ConfigOption (Description="Remove an excluded resource", Params=2, ParamsDescription="A type [path/pattern/mailfolder], a path/pattern/name")]
+			internal bool DelExclude (out string output, string [] args)
+			{
+				ExcludeType type;
+				try {
+					type = (ExcludeType) Enum.Parse (typeof (ExcludeType), args [0], true);
+				} catch (Exception e) {
+					output = String.Format("Invalid type '{0}'. Valid types: Path, Pattern, MailFolder", args [0]);
+					return false;
+				}
+
+				foreach (ExcludeItem item in excludes) {
+					if (item.Type != type || item.Value != args [1])
+						continue;
+					excludes.Remove (item);
+					output = "Exclude removed.";
+					return true;
+				}
+
+				output = "Could not find requested exclude to remove.";
+				return false;
+			}
+
 		}
 
-#if ENABLE_WEBSERVICES
+//#if ENABLE_WEBSERVICES
 		[ConfigSection (Name="webservices")]
 		public class WebServicesConfig: Section 
 		{
@@ -467,16 +527,27 @@ namespace Beagle.Util {
 
 				return true;
 			}
+			
+			[ConfigOption (Description="Check current configuration of global access to Beagle web-services", IsMutator=false)]
+			internal bool CheckGlobalAccess(out string output, string [] args)
+			{
+				if (allowGlobalAccess)
+					output = "Global Access to Beagle WebServices is currently ENABLED.";
+				else
+					output = "Global Access to Beagle WebServices is currently DISABLED.";
 
+				return true;
+			}
+			
 			[ConfigOption (Description="Enable/Disable global access to Beagle web-services")]
 			internal bool SwitchGlobalAccess (out string output, string [] args)
 			{
 				allowGlobalAccess = !allowGlobalAccess;			
 				
 				if (allowGlobalAccess)
-					output = "Global Access to Beagle WebService ENABLED.";
+					output = "Global Access to Beagle WebServices now ENABLED.";
 				else
-					output = "Global Access to Beagle WebService DISABLED.";
+					output = "Global Access to Beagle WebServices now DISABLED.";
 
 				return true;
 			}
@@ -484,16 +555,16 @@ namespace Beagle.Util {
 			[ConfigOption (Description="Add public web-service access to a folder", Params=1, ParamsDescription="A path")]
 			internal bool AddPublicFolder (out string output, string [] args)
 			{
-				publicFolders.Add (args [0]);
-				output = "PublicFolder added.";
+				publicFolders.Add (args [0]);					
+				output = "PublicFolder " + args[0] + " added.";
 				return true;
 			}
 
 			[ConfigOption (Description="Remove public web-service access to a folder", Params=1, ParamsDescription="A path")]
 			internal bool DelPublicFolder (out string output, string [] args)
 			{
-				publicFolders.Remove (args [0]);
-				output = "PublicFolder removed.";
+				publicFolders.Remove (args [0]);		
+				output = "PublicFolder " + args[0] + " removed.";
 				return true;
 			}			
 		}
@@ -524,20 +595,30 @@ namespace Beagle.Util {
 			[ConfigOption (Description="Add a Networked Beagle Daemon to query", Params=1, ParamsDescription="HostName:PortNo")]
 			internal bool AddBeagleNode (out string output, string [] args)
 			{
-				netBeagleNodes.Add (args [0]);
-				output = "Networked Beagle Daemon \"" + args[0] +"\" added.";
+				string node = args[0];
+				
+				if (((string[])node.Split(':')).Length < 2)
+					node = args [0].Trim() + ":8888";
+							
+				netBeagleNodes.Add(node);			
+				output = "Networked Beagle Daemon \"" + node +"\" added.";
 				return true;
 			}
 
 			[ConfigOption (Description="Remove a configured Networked Beagle Daemon", Params=1, ParamsDescription="HostName:PortNo")]
 			internal bool DelBeagleNode (out string output, string [] args)
 			{
-				netBeagleNodes.Remove (args [0]);
-				output = "Networked Beagle Daemon \"" + args[0] +"\" removed.";
+				string node = args[0];
+				
+				if (((string[])node.Split(':')).Length < 2)
+					node = args [0].Trim() + ":8888";
+							
+				netBeagleNodes.Remove(node);					
+				output = "Networked Beagle Daemon \"" + node +"\" removed.";
 				return true;
 			}
 		}
-#endif
+//#endif
 
 		public class Section {
 			[XmlIgnore]
@@ -655,6 +736,12 @@ namespace Beagle.Util {
 			ExcludeItem exclude = obj as ExcludeItem;
 			return (exclude != null && exclude.Type == type && exclude.Value == val);
 		}
+
+		public override int GetHashCode ()
+		{
+			return (this.Value.GetHashCode () ^ (int) this.Type);
+		}
+
 	}
 
 	//////////////////////////////////////////////////////////////////////
