@@ -27,22 +27,59 @@
 using System;
 using System.Collections;
 using System.IO;
+using System.Runtime.InteropServices;
+using System.Text;
 
 namespace Beagle.Util {
 
 	public class DirectoryWalker {
+#if false
+		public static void Main (string[] args)
+		{
+			IEnumerable en = DirectoryWalker.GetFileInfos (args [0]);
 
+			foreach (FileInfo file in en) {
+				Console.WriteLine (file.FullName);
+			}
+		}
+#endif
 		private delegate bool   FileFilter      (string path, string name);
 		private delegate object FileObjectifier (string path, string name);
 
-		private class FileEnumerator : IEnumerator {
+		[DllImport ("libc", SetLastError = true)]
+		private static extern IntPtr opendir (string name);
+		
+		[DllImport ("libc", SetLastError = true)]
+		private static extern int closedir (IntPtr dir);
 
+		[DllImport ("libbeagledglue", EntryPoint = "beagled_utils_readdir", SetLastError = true)]
+		private static extern int sys_readdir (IntPtr dir, StringBuilder name);
+		
+		private static string readdir (IntPtr dir)
+		{
+			int r = 0;
+			StringBuilder name = new StringBuilder (256);
+
+			while (r == 0 && name.Length == 0) {
+			       r = sys_readdir (dir, name); 
+			}
+
+			if (r == -1)
+				return null;
+
+			return name.ToString ();
+		}
+
+		private class FileEnumerator : IEnumerator {
+			
 			string path;
 			FileFilter file_filter;
 			FileObjectifier file_objectifier;
 			IntPtr dir_handle = IntPtr.Zero;
 			string current;
 
+			public bool NamesOnly = false;
+			
 			public FileEnumerator (string          path,
 					       FileFilter      file_filter,
 					       FileObjectifier file_objectifier)
@@ -52,11 +89,11 @@ namespace Beagle.Util {
 				this.file_objectifier = file_objectifier;
 				Reset ();
 			}
-
+			
 			~FileEnumerator ()
 			{
 				if (dir_handle != IntPtr.Zero)
-					Mono.Posix.Syscall.closedir (dir_handle);
+					closedir (dir_handle);
 			}
 
 			public object Current {
@@ -65,19 +102,22 @@ namespace Beagle.Util {
 					if (current != null) {
 						if (file_objectifier != null)
 							current_obj = file_objectifier (path, current); 
+						else if (NamesOnly)
+							current_obj = current;
 						else
 							current_obj = Path.Combine (path, current);
 					}
+
 					return current_obj;
 				}
 			}
 
 			public bool MoveNext ()
 			{
-				bool skip_file;
+				bool skip_file = false;
 
 				do {
-					current = Mono.Posix.Syscall.readdir (dir_handle);
+					current = readdir (dir_handle);
 					if (current == null)
 						break;
 
@@ -104,7 +144,7 @@ namespace Beagle.Util {
 				} while (skip_file);
 
 				if (current == null) {
-					Mono.Posix.Syscall.closedir (dir_handle);
+					closedir (dir_handle);
 					dir_handle = IntPtr.Zero;
 				}
 
@@ -115,8 +155,8 @@ namespace Beagle.Util {
 			{
 				current = null;
 				if (dir_handle != IntPtr.Zero)
-					Mono.Posix.Syscall.closedir (dir_handle);
-				dir_handle = Mono.Posix.Syscall.opendir (path);
+					closedir (dir_handle);
+				dir_handle = opendir (path);
 				if (dir_handle == IntPtr.Zero)
 					throw new DirectoryNotFoundException (path);
 			}
@@ -127,6 +167,8 @@ namespace Beagle.Util {
 			string path;
 			FileFilter file_filter;
 			FileObjectifier file_objectifier;
+			
+			public bool NamesOnly = false;
 
 			public FileEnumerable (string          path,
 					       FileFilter      file_filter,
@@ -139,7 +181,10 @@ namespace Beagle.Util {
 
 			public IEnumerator GetEnumerator ()
 			{
-				return new FileEnumerator (path, file_filter, file_objectifier);
+				FileEnumerator e;
+				e = new FileEnumerator (path, file_filter, file_objectifier);
+				e.NamesOnly = this.NamesOnly;
+				return e;
 			}
 		}
 
@@ -153,6 +198,20 @@ namespace Beagle.Util {
 			return new FileInfo (Path.Combine (path, name));
 		}
 
+		/////////////////////////////////////////////////////////////////////////////////
+
+		static public bool IsWalkable (string path)
+		{
+			IntPtr dir_handle;
+			dir_handle = opendir (path);
+			if (dir_handle == IntPtr.Zero)
+				return false;
+			closedir (dir_handle);
+			return true;
+		}
+
+		/////////////////////////////////////////////////////////////////////////////////
+
 		static public IEnumerable GetFiles (string path)
 		{
 			return new FileEnumerable (path, new FileFilter (IsFile), null);
@@ -195,6 +254,14 @@ namespace Beagle.Util {
 			return GetDirectories (dirinfo.FullName);
 		}
 
+		static public IEnumerable GetDirectoryNames (string path)
+		{
+			FileEnumerable fe;
+			fe = new FileEnumerable (path, new FileFilter (IsDirectory), null);
+			fe.NamesOnly = true;
+			return fe;
+		}
+
 		static public IEnumerable GetDirectoryInfos (string path)
 		{
 			return new FileEnumerable (path,
