@@ -102,26 +102,24 @@ namespace Beagle.Daemon {
 			usage +=
 				"Usage: beagled [OPTIONS]\n\n" +
 				"Options:\n" +
-				"  --foreground\t\tRun the daemon in the foreground.\n" +
-				"  --fg\t\t\tRun the daemon in the foreground.\n" +
-				"  --background\t\tRun the daemon in the background.\n" +
-				"  --bg\t\t\tRun the daemon in the background.\n" +
+				"  --foreground, --fg\tRun the daemon in the foreground.\n" +
+				"  --background, --bg\tRun the daemon in the background.\n" +
 				"  --replace\t\tReplace a running daemon with a new instance.\n" +
 				"  --debug\t\tWrite out debugging information.\n" +
 				"  --debug-memory\tWrite out debugging information about memory use.\n" +
 				"  --deny-backend\tDeny a specific backend.\n" +
 				"  --allow-backend\tAllow a specific backend.\n" +
 				"  --list-backends\tList all the available backends.\n" +
+				"  --add-static-backend\tAdd a static backend by path.\n" + 
 				"  --disable-scheduler\tDisable the use of the scheduler.\n" +
 				"  --help\t\tPrint this usage message.\n";
 
 #if ENABLE_WEBSERVICES
 			usage += "\n" +
 				"  --web-global\t\tAllow global access to the Web & WebService interfaces.\n" +
-				"  --web-start\t\tStart internal web server for this instance.\n" +
 				"  --web-port\t\tPort to use for the internal web server.\n" +
 				"  --web-root\t\tRoot directory to use for the internal web server.\n" +
-				"  --enable-network\tStart internal web server & allow global access.\n";
+				"  --web-disable\t\tDisable Web & WebServices functionality.\n";			
 #endif 
 
 			Console.WriteLine (usage);
@@ -151,6 +149,12 @@ namespace Beagle.Daemon {
 			if (Environment.GetEnvironmentVariable ("BEAGLE_ENABLE_IN_PROCESS_INDEXING") == null)
 				LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
 
+			// Initialize syncronization to keep the indexes local if PathFinder.HomeDir
+			// is on a non-block device, or if BEAGLE_SYNCHRONIZE_LOCALLY is set
+			if ((! SystemInformation.IsPathOnBlockDevice (PathFinder.HomeDir) && Conf.Daemon.IndexSynchronization) ||
+			    Environment.GetEnvironmentVariable ("BEAGLE_SYNCHRONIZE_LOCALLY") != null)
+				IndexSynchronization.Initialize ();
+
 			// Start the query driver.
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
@@ -174,8 +178,6 @@ namespace Beagle.Daemon {
 #endif
 			Shutdown.ShutdownEvent += OnShutdown;
 
-			// Load user configuration
-			Conf.Load ();
 			Conf.WatchForUpdates ();
 
 			stopwatch.Stop ();
@@ -247,16 +249,26 @@ namespace Beagle.Daemon {
 					++i; // we used next_arg
 					break;
 
+			       case "--add-static-backend": 
+					if (next_arg != null)
+						QueryDriver.AddStaticQueryable (next_arg);
+					++i;
+					break;
+
 				case "--disable-scheduler":
 					arg_disable_scheduler = true;
 					break;
+
+				case "--autostarted":
+					if (! Conf.Searching.Autostart) {
+						Console.WriteLine ("Autostarting is disabled, not starting");
+						Environment.Exit (0);
+					}
+					break;
 #if ENABLE_WEBSERVICES
 				case "--web-global":
 					WebServiceBackEnd.web_global = true;
-					break;
-
-				case "--web-start":
-					WebServiceBackEnd.web_start = true;
+					WebServiceBackEnd.web_start = true;					
 					break;
 
 				case "--web-port":
@@ -271,10 +283,9 @@ namespace Beagle.Daemon {
 					WebServiceBackEnd.web_start = true;
 					break;
 					
-				case "--enable-network":
-					WebServiceBackEnd.web_start = true;
-					WebServiceBackEnd.web_global = true;
-					break;					
+				case "--web-disable":
+					WebServiceBackEnd.web_start = false;
+					break;				
 #endif 
 				default:
 					Console.WriteLine ("Ignoring unknown argument '{0}'", arg);
@@ -308,17 +319,9 @@ namespace Beagle.Daemon {
 						  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
 			} catch (Exception ex) { }
 
-			// Make sure that extended attributes can be set.  If not, bail out with a message.
-			// FIXME FIXME FIXME: This assumes that EAs work the same on your storage dir
-			// as they do on your home dir, which is obviously not a good assumption.
-			// We just need to pick a file under the home dir with the right permissions
-			// and test it.
-			if (! ExtendedAttribute.Test (PathFinder.StorageDir)) {
-				Logger.Log.Fatal ("Could not set extended attributes on a file in your home "
-						  + "directory.  See "
-						  + "http://www.beaglewiki.org/Enabling_Extended_Attributes "
-						  + "for more information.");
-				return 1;
+			if (! ExtendedAttribute.Supported) {
+				Logger.Log.Warn ("Extended attributes are not supported on this filesystem.  " +
+						 "Many search backends will not be available");
 			}
 
 			// Start our memory-logging thread
@@ -340,6 +343,14 @@ namespace Beagle.Daemon {
 
 			Application.Run ();
 
+			// If we placed our sockets in a temp directory, try to clean it up
+			// Note: this may fail because the helper is still running
+			if (PathFinder.GetRemoteStorageDir (false) != PathFinder.StorageDir) {
+				try {
+					Directory.Delete (PathFinder.GetRemoteStorageDir (false));
+				} catch (IOException) { }
+			}
+
 			Logger.Log.Debug ("Leaving BeagleDaemon.Main");
 
 			if (Logger.Log.Level == LogLevel.Debug) {
