@@ -70,6 +70,11 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		public abstract Indexable GetNextIndexable ();
 		public abstract void Checkpoint ();
 
+		public void PostFlushHook ()
+		{
+			Checkpoint ();
+		}
+
 		protected EvolutionMailQueryable Queryable {
 			get { return this.queryable; }
 		}
@@ -102,7 +107,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					
 		protected void CrawlFinished ()
 		{
-			this.queryable.FileAttributesStore.AttachTimestamp (this.CrawlFile.FullName, DateTime.Now);
+			// FIXME: This is a little sketchy
+			this.queryable.FileAttributesStore.AttachLastWriteTime (this.CrawlFile.FullName, DateTime.Now);
 		}
 
 		public string StatusName {
@@ -136,6 +142,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		private int mbox_fd = -1;
 		private GMime.StreamFs mbox_stream;
 		private GMime.Parser mbox_parser;
+		private long file_size;
 
 		public EvolutionMailIndexableGeneratorMbox (EvolutionMailQueryable queryable, FileInfo mbox_info) : base (queryable)
 		{
@@ -181,7 +188,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				string offset_str = this.queryable.ReadDataLine ("offset-" + this.folder_name.Replace ('/', '-'));
 				long offset = Convert.ToInt64 (offset_str);
 
-				Logger.Log.Debug ("mbox {0} offset is {1}", this.mbox_info.Name, offset);
 				return offset;
 			}
 
@@ -198,7 +204,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			}
 
 			if (this.mbox_fd < 0) {
-				Logger.Log.Debug ("opening mbox {0}", this.mbox_info.Name);
+				Logger.Log.Debug ("Opening mbox {0}", this.mbox_info.Name);
 
 				try {
 					InitializeGMime ();
@@ -213,6 +219,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				this.mbox_stream.Seek ((int) this.MboxLastOffset);
 				this.mbox_parser = new GMime.Parser (this.mbox_stream);
 				this.mbox_parser.ScanFrom = true;
+
+				FileInfo info = new FileInfo (this.mbox_info.FullName);
+				this.file_size = info.Length;
 			}
 
 			if (this.mbox_parser.Eos ()) {
@@ -226,8 +235,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				this.mbox_parser.Dispose ();
 				this.mbox_parser = null;
 				
-				EvolutionMailQueryable.log.Debug ("{0}: Finished indexing {1} messages",
-								  this.folder_name, this.indexed_count);
+				Logger.Log.Debug ("{0}: Finished indexing {1} messages", this.folder_name, this.indexed_count);
 
 				this.MboxLastOffset = offset;
 				this.CrawlFinished ();
@@ -257,8 +265,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				
 				string x_evolution = message.GetHeader ("X-Evolution");
 				if (x_evolution == null || x_evolution == "") {
-					EvolutionMailQueryable.log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
-									 this.folder_name, this.mbox_parser.FromOffset);
+					Logger.Log.Info ("{0}: Message at offset {1} has no X-Evolution header!",
+							 this.folder_name, this.mbox_parser.FromOffset);
 					return null;
 				}
 				
@@ -275,9 +283,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 				++this.indexed_count;
 
-				// HACK: update your recipients
-				EvolutionMailQueryable.AddAsYourRecipient (indexable);
-				
 				return indexable;
 			}
 		}
@@ -297,7 +302,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Indexable indexable = new Indexable (uri);
 
 			indexable.Timestamp = message.Date;
-			indexable.Type = "MailMessage";
+			indexable.HitType = "MailMessage";
 			indexable.MimeType = "message/rfc822";
 			indexable.CacheContent = false;
 
@@ -331,10 +336,13 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
+			//indexable.AddProperty (Property.NewDate ("fixme:date", message.Date));
+#if false
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewDate ("fixme:sentdate", message.Date));
 			else
 				indexable.AddProperty (Property.NewDate ("fixme:received", message.Date));
+#endif
 
 			indexable.AddProperty (Property.NewKeyword ("fixme:flags", flags));
 
@@ -366,11 +374,19 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public override void Checkpoint ()
 		{
-			EvolutionMailQueryable.log.Debug ("{0}: indexed {1} messages",
-							  this.folder_name, this.indexed_count);
+			long offset = -1;
 
 			if (this.mbox_parser != null)
-				this.MboxLastOffset = this.mbox_parser.FromOffset;
+				this.MboxLastOffset = offset = this.mbox_parser.FromOffset;
+
+			string progress = "";
+			if (this.file_size > 0 && offset > 0) {
+				progress = String.Format (" ({0}/{1} bytes {2:###.0}%)",
+							  offset, this.file_size, 100.0 * offset / this.file_size);
+			}
+
+			Logger.Log.Debug ("{0}: indexed {1} messages{2}",
+					  this.folder_name, this.indexed_count, progress);
 		}
 
 		public override string GetTarget ()
@@ -444,8 +460,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			try {
 				this.accounts = (ICollection) GConfThreadHelper.Get ("/apps/evolution/mail/accounts");
 			} catch (Exception ex) {
-				EvolutionMailQueryable.log.Warn ("Caught exception in Setup(): " + ex.Message);
-				EvolutionMailQueryable.log.Warn ("There are no configured evolution accounts, ignoring {0}", this.imap_name);
+				Logger.Log.Warn ("Caught exception in Setup(): " + ex.Message);
+				Logger.Log.Warn ("There are no configured evolution accounts, ignoring {0}", this.imap_name);
 				return false;
 			}
 
@@ -490,6 +506,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (semicolon != -1)
 					imap_url = imap_url.Substring (0, semicolon) + imap_url.Substring (user_end);
 
+				// Escape backslashes, which frequently appear when using IMAP against Exchange servers
+				this.imap_name = this.imap_name.Replace ("\\", "%5c");
+
 				// Escape out additional @s in the name.  I hate the class libs so much.
 				int lastIdx = this.imap_name.LastIndexOf ('@');
 				if (this.imap_name.IndexOf ('@') != lastIdx) {
@@ -509,8 +528,8 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				}
 			}
 
-			if (account_name == null) {
-				EvolutionMailQueryable.log.Info ("Unable to determine account name for {0}", this.imap_name);
+			if (this.account_name == null) {
+				Logger.Log.Info ("Unable to determine account name for {0}", this.imap_name);
 				return false;
 			}
 
@@ -531,6 +550,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		private string FolderCacheName {
 			get {
+				if (this.account_name == null || this.folder_name == null)
+					return null;
+
 				if (this.folder_cache_name == null)
 					this.folder_cache_name = "status-" + this.account_name + "-" + this.folder_name.Replace ('/', '-');
 
@@ -543,12 +565,17 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Stream cacheStream;
 			BinaryFormatter formatter;
 
+			if (this.FolderCacheName == null) {
+				this.mapping = new Hashtable ();
+				return false;
+			}
+
 			try {
 				cacheStream = this.queryable.ReadDataStream (this.FolderCacheName);
 				formatter = new BinaryFormatter ();
 				this.mapping = formatter.Deserialize (cacheStream) as Hashtable;
 				cacheStream.Close ();
-				EvolutionMailQueryable.log.Debug ("Successfully loaded previous crawled data from disk: {0}", this.FolderCacheName);
+				Logger.Log.Debug ("Successfully loaded previous crawled data from disk: {0}", this.FolderCacheName);
 
 				return true;
 			} catch {
@@ -562,6 +589,9 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		{
 			Stream cacheStream;
 			BinaryFormatter formatter;
+
+			if (this.FolderCacheName == null)
+				return;
 			
 			cacheStream = this.queryable.WriteDataStream (this.FolderCacheName);
 			formatter = new BinaryFormatter ();
@@ -585,7 +615,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 				// Check to see if we even need to bother walking the summary
 				if (cache_loaded && this.queryable.FileAttributesStore.IsUpToDate (this.CrawlFile.FullName)) {
-					EvolutionMailQueryable.log.Debug ("{0}: summary has not been updated; crawl unncessary", this.folder_name);
+					Logger.Log.Debug ("{0}: summary has not been updated; crawl unncessary", this.folder_name);
 					return false;
 				}
 			}
@@ -597,7 +627,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 					else
 						this.summary = Camel.Summary.LoadImap4Summary (this.summary_info.FullName);
 				} catch (Exception e) {
-					EvolutionMailQueryable.log.Warn ("Unable to index {0}: {1}", this.folder_name,
+					Logger.Log.Warn ("Unable to index {0}: {1}", this.folder_name,
 									 e.Message);
 					return false;
 				}
@@ -620,10 +650,15 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				this.Queryable.ThisScheduler.Add (task);
 			}
 
-			EvolutionMailQueryable.log.Debug ("{0}: Finished indexing {1} ({2}/{3} {4:###.0}%)",
-							  this.folder_name, this.indexed_count, this.count,
+			string progress = "";
+			if (this.count > 0 && this.summary.header.count > 0) {
+				progress = String.Format (" ({0}/{1} {2:###.0}%)",
+							  this.count,
 							  this.summary.header.count,
 							  100.0 * this.count / this.summary.header.count);
+			}
+
+			Logger.Log.Debug ("{0}: Finished indexing {1} messages{2}", this.folder_name, this.indexed_count, progress);
 
 			this.SaveCache ();
 			this.CrawlFinished ();
@@ -669,9 +704,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (indexable != null) {
 					++this.indexed_count;
 					this.deleted_list.Remove (mi.uid);
-					
-					// HACK: update your recipients
-					EvolutionMailQueryable.AddAsYourRecipient (indexable);
 				}
 			} else if (flags != null)
 				this.deleted_list.Remove (mi.uid);
@@ -700,13 +732,16 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			indexable.Timestamp = messageInfo.Date;
 			indexable.MimeType = "message/rfc822";
-			indexable.Type = "MailMessage";
+			indexable.HitType = "MailMessage";
 
 			indexable.AddProperty (Property.NewKeyword ("fixme:account",  this.imap_name));
                         indexable.AddProperty (Property.NewKeyword ("fixme:folder",   this.folder_name));
+			indexable.AddProperty (Property.NewKeyword ("fixme:client", "evolution"));
 			
-			if (!have_content)
+			if (!have_content) {
 				indexable.AddProperty (Property.New ("dc:title", messageInfo.subject));
+				indexable.AddProperty (Property.NewDate ("fixme:date", messageInfo.Date));
+			}
 
 			GMime.InternetAddressList addrs;
 			addrs = GMime.InternetAddressList.ParseString (messageInfo.to);
@@ -714,7 +749,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:to", ia.ToString (false)));
 					indexable.AddProperty (Property.NewKeyword ("fixme:to_address", ia.Addr));
-					indexable.AddProperty (Property.NewKeyword ("fixme:to_name", ia.Name));
+					indexable.AddProperty (Property.New ("fixme:to_name", ia.Name));
 				}
 
 				if (this.folder_name == "Sent")
@@ -727,7 +762,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:cc", ia.ToString (false)));
 					indexable.AddProperty (Property.NewKeyword ("fixme:cc_address", ia.Addr));
-					indexable.AddProperty (Property.NewKeyword ("fixme:cc_name", ia.Name));
+					indexable.AddProperty (Property.New ("fixme:cc_name", ia.Name));
 				}
 
 				if (this.folder_name == "Sent")
@@ -740,7 +775,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 				if (!have_content) {
 					indexable.AddProperty (Property.NewKeyword ("fixme:from", ia.ToString (false)));
 					indexable.AddProperty (Property.NewKeyword ("fixme:from_address", ia.Addr));
-					indexable.AddProperty (Property.NewKeyword ("fixme:from_name", ia.Name));
+					indexable.AddProperty (Property.New ("fixme:from_name", ia.Name));
 				}
 
 				if (this.folder_name != "Sent")
@@ -751,12 +786,6 @@ namespace Beagle.Daemon.EvolutionMailDriver {
                         indexable.AddProperty (Property.NewKeyword ("fixme:mlist",    messageInfo.mlist));
                         indexable.AddProperty (Property.NewKeyword ("fixme:flags",    messageInfo.flags));
 
-			if (messageInfo.received != DateTime.MinValue)
-				indexable.AddProperty (Property.NewDate ("fixme:received", messageInfo.received));
-
-			if (messageInfo.sent != DateTime.MinValue)
-				indexable.AddProperty (Property.NewDate ("fixme:sentdate", messageInfo.sent));
-
 			if (this.folder_name == "Sent")
 				indexable.AddProperty (Property.NewFlag ("fixme:isSent"));
 
@@ -783,7 +812,7 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 			if (have_content)
 				indexable.ContentUri = UriFu.PathToFileUri (msg_file);
-			else
+			else 
 				indexable.NoContent = true;
 
 			return indexable;
@@ -791,10 +820,20 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 
 		public override void Checkpoint ()
 		{
-			EvolutionMailQueryable.log.Debug ("{0}: indexed {1} messages ({2}/{3} {4:###.0}%)",
-							  this.folder_name, this.indexed_count, this.count,
-							  this.summary.header.count,
-							  100.0 * this.count / this.summary.header.count);
+			if (this.summary != null) {
+
+				string progress = "";
+				if (this.count > 0 && this.summary.header.count > 0) {
+					progress = String.Format (" ({0}/{1} {2:###.0}%)",
+								  this.count,
+								  this.summary.header.count,
+								  100.0 * this.count / this.summary.header.count);
+
+				}
+				
+				Logger.Log.Debug ("{0}: indexed {1} messages{2}",
+						  this.folder_name, this.indexed_count, progress);
+			}
 
 			this.SaveCache ();
 		}
