@@ -35,7 +35,7 @@ using Beagle.Util;
 
 namespace Beagle.Daemon.GaimLogQueryable {
 
-	[QueryableFlavor (Name="IMLog", Domain=QueryDomain.Local, RequireInotify=false)]
+	[QueryableFlavor (Name="GaimLog", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class GaimLogQueryable : LuceneFileQueryable {
 
 		private static Logger log = Logger.Get ("GaimLogQueryable");
@@ -78,6 +78,7 @@ namespace Beagle.Daemon.GaimLogQueryable {
 			if (!Inotify.Enabled) {
 				Scheduler.Task task = Scheduler.TaskFromHook (new Scheduler.TaskHook (CrawlHook));
                                 task.Tag = "Crawling ~/.gaim/logs to find new logfiles";
+				task.Source = this;
                                 ThisScheduler.Add (task);
 			}
 
