@@ -38,17 +38,37 @@ namespace Beagle.Daemon {
 
 	public class TextCache {
 
-		const string SELF_CACHE_TAG = "*self*";
+		static public bool Debug = false;
 
-		static string text_cache_dir;
-		static SqliteConnection connection;
+		public const string SELF_CACHE_TAG = "*self*";
 
-		static TextCache () 
+		private string text_cache_dir;
+		private SqliteConnection connection;
+
+		private enum TransactionState {
+			None,
+			Requested,
+			Started
+		}
+		private TransactionState transaction_state;
+
+		private static TextCache user_cache = null;
+
+		public static TextCache UserCache {
+			get {
+				if (user_cache == null)
+					user_cache = new TextCache (PathFinder.StorageDir);
+				
+				return user_cache;
+			}
+		}
+
+		public TextCache (string storage_dir) 
 		{
-			text_cache_dir = Path.Combine (PathFinder.StorageDir, "TextCache");
+			text_cache_dir = Path.Combine (storage_dir, "TextCache");
 			if (! Directory.Exists (text_cache_dir)) {
 				Directory.CreateDirectory (text_cache_dir);
-
+				
 				// Create our cache subdirectories.
 				for (int i = 0; i < 256; ++i) {
 					string subdir = i.ToString ("x");
@@ -66,7 +86,8 @@ namespace Beagle.Daemon {
 				create_new_db = true;
 			
 			connection = new SqliteConnection ();
-			connection.ConnectionString = "URI=file:" + db_filename;
+			connection.ConnectionString = "version=" + ExternalStringsHack.SqliteVersion
+					+ ",URI=file:" + db_filename;
 			connection.Open ();
 			
 			if (create_new_db) {
@@ -82,7 +103,7 @@ namespace Beagle.Daemon {
 			return uri.ToString ().Replace ("'", "''");
 		}
 
-		private static SqliteCommand NewCommand (string format, params object [] args)
+		private SqliteCommand NewCommand (string format, params object [] args)
 		{
 			SqliteCommand command;
 			command = new SqliteCommand ();
@@ -91,8 +112,10 @@ namespace Beagle.Daemon {
 			return command;
 		}
 
-		private static void DoNonQuery (string format, params object [] args)
+		private void DoNonQuery (string format, params object [] args)
 		{
+			if (Debug)
+				Logger.Log.Debug ("Executing command '{0}'", String.Format (format, args));
 			SqliteCommand command = NewCommand (format, args);
 			while (true) {
 				try {
@@ -109,7 +132,7 @@ namespace Beagle.Daemon {
 			command.Dispose ();
 		}
 
-		static private SqliteDataReader ExecuteReaderOrWait (SqliteCommand command)
+		private SqliteDataReader ExecuteReaderOrWait (SqliteCommand command)
 		{
 			SqliteDataReader reader = null;
 			while (reader == null) {
@@ -125,7 +148,7 @@ namespace Beagle.Daemon {
 			return reader;
 		}
 
-		static private bool ReadOrWait (SqliteDataReader reader)
+		private bool ReadOrWait (SqliteDataReader reader)
 		{
 			while (true) {
 				try {
@@ -139,13 +162,16 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		private static void Insert (Uri uri, string filename)
+		private void Insert (Uri uri, string filename)
 		{
-			DoNonQuery ("INSERT OR REPLACE INTO uri_index (uri, filename) VALUES ('{0}', '{1}')",
-				    UriToString (uri), filename);
+			lock (connection) {
+				MaybeStartTransaction_Unlocked ();
+				DoNonQuery ("INSERT OR REPLACE INTO uri_index (uri, filename) VALUES ('{0}', '{1}')",
+					    UriToString (uri), filename);
+			}
 		}
 
-		private static string LookupPathRawUnlocked (Uri uri, bool create_if_not_found)
+		private string LookupPathRawUnlocked (Uri uri, bool create_if_not_found)
 		{
 			SqliteCommand command;
 			SqliteDataReader reader = null;
@@ -166,20 +192,32 @@ namespace Beagle.Daemon {
 			}
 
 			if (path == SELF_CACHE_TAG)
-				return path;
+				return SELF_CACHE_TAG;
 
 			return path != null ? Path.Combine (text_cache_dir, path) : null;
 		}
-	
-		private static string LookupPath (Uri uri,
-						  LuceneDriver.UriRemapper uri_remapper,
-						  bool create_if_not_found)
+
+		// Don't do this unless you know what you are doing!  If you
+		// do anything to the path you get back other than open and
+		// read the file, you will almost certainly break something.
+		// And it will be evidence that you are a bad person and that
+		// you deserve whatever horrible fate befalls you.
+		public string LookupPathRaw (Uri uri)
+		{
+			lock (connection)
+				return LookupPathRawUnlocked (uri, false);
+		}
+
+		private string LookupPath (Uri uri, bool create_if_not_found)
 		{
 			lock (connection) {
 				string path = LookupPathRawUnlocked (uri, create_if_not_found);
 				if (path == SELF_CACHE_TAG) {
+					// FIXME: How do we handle URI remapping for self-cached items?
+#if false
 					if (uri_remapper != null)
 						uri = uri_remapper (uri);
+#endif
 					if (! uri.IsFile) {
 						string msg = String.Format ("Non-file uri {0} flagged as self-cached", uri);
 						throw new Exception (msg);
@@ -189,16 +227,17 @@ namespace Beagle.Daemon {
 				return path;
 			}
 		}
-
-		public static void MarkAsSelfCached (Uri uri)
+		
+		public void MarkAsSelfCached (Uri uri)
 		{
 			lock (connection)
 				Insert (uri, SELF_CACHE_TAG);
 		}
 
-		public static TextWriter GetWriter (Uri uri)
+		public TextWriter GetWriter (Uri uri)
 		{
-			string path = LookupPath (uri, null, true);
+			// FIXME: Uri remapping?
+			string path = LookupPath (uri, true);
 
 			FileStream stream;
 			stream = new FileStream (path, FileMode.Create, FileAccess.Write, FileShare.ReadWrite);
@@ -211,9 +250,33 @@ namespace Beagle.Daemon {
 			return writer;
 		}
 
-		public static TextReader GetReader (Uri uri, LuceneDriver.UriRemapper uri_remapper)
+		public void WriteFromReader (Uri uri, TextReader reader)
 		{
-			string path = LookupPath (uri, uri_remapper, false);
+			TextWriter writer;
+			writer = GetWriter (uri);
+			string line;
+			while ((line = reader.ReadLine ()) != null)
+				writer.WriteLine (line);
+			writer.Close ();
+		}
+
+		public void WriteFromString (Uri uri, string str)
+		{
+			if (str == null) {
+				Delete (uri);
+				return;
+			}
+
+			TextWriter writer;
+			writer = GetWriter (uri);
+			writer.WriteLine (str);
+			writer.Close ();
+		}
+
+		// FIXME: Uri remapping?
+		public TextReader GetReader (Uri uri)
+		{
+			string path = LookupPath (uri, false);
 			if (path == null)
 				return null;
 
@@ -229,11 +292,12 @@ namespace Beagle.Daemon {
 			return reader;
 		}
 
-		public static void Delete (Uri uri)
+		public void Delete (Uri uri)
 		{
 			lock (connection) {
 				string path = LookupPathRawUnlocked (uri, false);
 				if (path != null) {
+					MaybeStartTransaction_Unlocked ();
 					DoNonQuery ("DELETE FROM uri_index WHERE uri='{0}' AND filename='{1}'", 
 					            UriToString (uri), path);
 					if (path != SELF_CACHE_TAG)
@@ -241,5 +305,29 @@ namespace Beagle.Daemon {
 				}
 			}
 		}
+
+		private void MaybeStartTransaction_Unlocked ()
+		{
+			if (transaction_state == TransactionState.Requested)
+				DoNonQuery ("BEGIN");
+			transaction_state = TransactionState.Started;
+		}
+
+		public void BeginTransaction ()
+		{
+			lock (connection) {
+				if (transaction_state == TransactionState.None)
+					transaction_state = TransactionState.Requested;
+			}
+		}
+
+		public void CommitTransaction ()
+		{
+			lock (connection) {
+				if (transaction_state == TransactionState.Started)
+					DoNonQuery ("COMMIT");
+				transaction_state = TransactionState.None;
+			}
+		}
 	}
 }
