@@ -44,12 +44,14 @@ namespace Beagle.Daemon {
 		static Logger log = Logger.Get ("NetworkedBeagle");
 	    static readonly string NetBeagleConfigFile = "netbeagle.cfg";
 		public static readonly string BeagleNetPrefix = "netbeagle://";
+		public static bool NetBeagleListActive = false;
 				
 		ArrayList NetBeagleList;
 		
 		public NetworkedBeagle ()
 		{
-			NetBeagleList = new ArrayList ();		
+			NetBeagleList = new ArrayList ();	
+			NetBeagleListActive = false;	
 		}
 
 /////////////////////////////////////////////////////////////////////////////////////////			
@@ -104,13 +106,14 @@ namespace Beagle.Daemon {
 					}
 					
 				if (NetBeagleList.Count > 0) {
-				if (File.Exists (Path.Combine (PathFinder.StorageDir, NetBeagleConfigFile)))
-				{
-					log.Warn("NetBeagleQueryable: Duplicate configuration of networked Beagles detected!");
-					log.Info("NetBeagleQueryable: Remove '~/.beagle/netbeagle.cfg' file. Use 'beagle-config' instead to setup networked Beagle nodes.");		
-					log.Info("Using ~/.beagle/config/networking.xml");	
-				}
-				return;
+					NetBeagleListActive = true;
+					if (File.Exists (Path.Combine (PathFinder.StorageDir, NetBeagleConfigFile)))
+					{
+						log.Warn("NetBeagleQueryable: Duplicate configuration of networked Beagles detected!");
+						log.Info("NetBeagleQueryable: Remove '~/.beagle/netbeagle.cfg' file. Use 'beagle-config' instead to setup networked Beagle nodes.");		
+						log.Info("Using ~/.beagle/config/networking.xml");	
+					}
+					return;
 				}
 			}
 			
@@ -136,9 +139,12 @@ namespace Beagle.Daemon {
 					NetBeagleList.Add (new NetBeagleHandler (host, port, this));					
 				}								
 			}
-			
-			//if (NetBeagleList.Count > 0)
-			//	log.Warn("NetBeagleQueryable: 'netbeagle.cfg' based configuration deprecated.\n Use 'beagle-config' or 'beagle-settings' instead to configure Networked Beagles");
+
+			if (NetBeagleList.Count > 0) {
+				NetBeagleListActive = true;
+				//log.Warn("NetBeagleQueryable: 'netbeagle.cfg' based configuration deprecated.\n Use 'beagle-config' or 'beagle-settings' instead to configure Networked Beagles");
+				log.Warn("NetBeagleQueryable: 'netbeagle.cfg' based configuration deprecated.\n Use 'beagle-config' instead to configure Networked Beagles");				
+			}
 		}
 
 		private void NetBeagleConfigurationChanged (Conf.Section section)
@@ -149,8 +155,8 @@ namespace Beagle.Daemon {
 				
 			Conf.NetworkingConfig nc = (Conf.NetworkingConfig) section;
 
-			if (nc.NetBeagleNodes.Count == 0)
-				return;
+			//if (nc.NetBeagleNodes.Count == 0)
+			//	return;
 					
 			ArrayList newList = new ArrayList();
 			foreach (string nb in nc.NetBeagleNodes) {
@@ -166,11 +172,12 @@ namespace Beagle.Daemon {
 					newList.Add (new NetBeagleHandler (host, port, this));												
 			}	
 			
-			lock (NetBeagleList) {
+			lock (this) {
 				NetBeagleList = newList;
+				NetBeagleListActive = (newList.Count == 0) ? false:true;
 			}			 
 		}
-				
+	
 		public bool AcceptQuery (Query query)
 		{      
 		    if (query.Text.Count <= 0)
@@ -301,8 +308,10 @@ namespace Beagle.Daemon {
 				
 				if (!cTimer.Enabled) {
 					cTimer.Start();		
-					log.Debug("CachedRequestCleanupTimer started");			
+					log.Info("CachedRequestCleanupTimer started");			
 				}
+				
+				log.Info("CacheRequest: HopCount = " + hops);
 			}		
 		}		
 			
@@ -337,11 +346,11 @@ namespace Beagle.Daemon {
 			}
 			
 			if ((c % 4) == 0)		//Log status every 1 minute
-				log.Debug("CachedRequestCleanupTimer-EventHandler: requestTable has {0} elements, Last entry count={1}", requestTable.Count, c);
+				log.Info("CachedRequestCleanupTimer-EventHandler: requestTable has {0} elements, Last entry count={1}", requestTable.Count, c);
 
 			if (timerTable.Count == 0) {				
 				cTimer.Stop();
-				log.Debug("Stopping CachedRequestCleanupTimer");
+				log.Info("Stopping CachedRequestCleanupTimer");
 			}			
 		}
 		
